@@ -23,6 +23,7 @@ static LightDMGreeter *greeter;
 static GKeyFile *state;
 static gchar *state_filename;
 static GtkWindow *login_window, *panel_window;
+static GtkButton *login_button, *cancel_button;
 static GtkLabel *message_label, *prompt_label;
 static GtkWidget *login_box, *prompt_box;
 static GtkEntry *prompt_entry;
@@ -143,6 +144,24 @@ set_message_label (const gchar *text)
     gtk_label_set_text (message_label, text);
 }
 
+static void
+set_login_button_label (const gchar *username)
+{
+        LightDMUser *user;
+        gboolean logged_in = FALSE;
+
+        user = lightdm_user_list_get_user_by_name (lightdm_user_list_get_instance (), username);
+        /* Show 'Unlock' instead of 'Login' for an already logged in user */
+        logged_in = user && lightdm_user_get_logged_in (user);
+        if (logged_in)
+            gtk_button_set_label (login_button, _("Unlock"));
+        else
+            gtk_button_set_label (login_button, _("Login"));
+        /* and disable the session and language comboboxes */
+        gtk_widget_set_sensitive (GTK_WIDGET (session_combo), !logged_in);
+        gtk_widget_set_sensitive (GTK_WIDGET (language_combo), !logged_in);
+}
+
 static void
 start_authentication (const gchar *username)
 {
@@ -169,6 +188,7 @@ start_authentication (const gchar *username)
 
     if (strcmp (username, "*other") == 0)
     {
+        gtk_widget_show (GTK_WIDGET (cancel_button));
         lightdm_greeter_authenticate (greeter, NULL);
     }
     else if (strcmp (username, "*guest") == 0)
@@ -198,6 +218,10 @@ start_authentication (const gchar *username)
 static void
 cancel_authentication (void)
 {
+    GtkTreeModel *model;
+    GtkTreeIter iter;
+    gboolean other = FALSE;
+
     /* If in authentication then stop that first */
     cancelling = FALSE;
     if (lightdm_greeter_get_in_authentication (greeter))
@@ -207,14 +231,23 @@ cancel_authentication (void)
         set_message_label ("");
     }
 
+    /* Force refreshing the prompt_box for "Other" */
+    model = gtk_combo_box_get_model (user_combo);
+
+    if (gtk_combo_box_get_active_iter (user_combo, &iter))
+    {
+        gchar *user;
+
+        gtk_tree_model_get (GTK_TREE_MODEL (model), &iter, 0, &user, -1);
+        other = (strcmp (user, "*other") == 0);
+        g_free (user);
+    }
+
     /* Start a new login or return to the user list */
-    if (lightdm_greeter_get_hide_users_hint (greeter))
+    if (other || lightdm_greeter_get_hide_users_hint (greeter))
         start_authentication ("*other");
     else
-    {
-        gtk_widget_hide (login_box);
         gtk_widget_grab_focus (GTK_WIDGET (user_combo));
-    }
 }
 
 static void
@@ -272,6 +305,13 @@ user_combobox_active_changed_cb (GtkComboBox *widget, LightDMGreeter *greeter)
         gchar *user;
 
         gtk_tree_model_get (GTK_TREE_MODEL (model), &iter, 0, &user, -1);
+
+        if (strcmp (user, "*other") == 0)
+            gtk_widget_show (GTK_WIDGET (cancel_button));
+        else
+            gtk_widget_hide (GTK_WIDGET (cancel_button));
+
+        set_login_button_label (user);
         start_authentication (user);
         g_free (user);
     }
@@ -634,6 +674,7 @@ load_user_list ()
             if (matched)
             {
                 gtk_combo_box_set_active_iter (user_combo, &iter);
+                set_login_button_label (selected_user);
                 start_authentication (selected_user);
                 break;
             }
@@ -845,11 +886,6 @@ main (int argc, char **argv)
         g_object_set (gtk_settings_get_default (), "gtk-xft-rgba", value, NULL);
     g_free (value);
 
-    /* Load out installed icons */
-    gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (), GREETER_DATA_DIR);
-    gchar **path;
-    gtk_icon_theme_get_search_path (gtk_icon_theme_get_default (), &path, NULL);
-
     builder = gtk_builder_new ();
     if (!gtk_builder_add_from_file (builder, GREETER_DATA_DIR "/greeter.ui", &error))
     {
@@ -860,6 +896,8 @@ main (int argc, char **argv)
 
     login_window = GTK_WINDOW (gtk_builder_get_object (builder, "login_window"));
     login_box = GTK_WIDGET (gtk_builder_get_object (builder, "login_box"));
+    login_button = GTK_BUTTON (gtk_builder_get_object (builder, "login_button"));
+    cancel_button = GTK_BUTTON (gtk_builder_get_object (builder, "cancel_button"));
     prompt_box = GTK_WIDGET (gtk_builder_get_object (builder, "prompt_box"));
     prompt_label = GTK_LABEL (gtk_builder_get_object (builder, "prompt_label"));
     prompt_entry = GTK_ENTRY (gtk_builder_get_object (builder, "prompt_entry"));
@@ -991,6 +1029,7 @@ main (int argc, char **argv)
     else
     {
         load_user_list ();
+        gtk_widget_hide (GTK_WIDGET (cancel_button));
         gtk_widget_show (GTK_WIDGET (user_combo));
     } 
 
